#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "forms.h"
#include "board_ui.h"
#include "board_proto.h"

#define P_RAD 6
#ifdef OLD_TABLE
#define TABLE_SIDE (8.0*25.4)  /* in mm ! */
#else
#define TABLE_SIDE (15.0*25.4) /* in mm ! */
#endif

#define TABLE_SIDE_OV2 (0.5*TABLE_SIDE)

typedef struct {
  _ushort x, y;
} struct_vec2d_ushort;

typedef struct {
  struct_vec2d pos, accel1, accel2;
} struct_sensor;

#ifdef OLD_TABLE
static struct_sensor g_s1 = {{-96.6,-92.6}};
static struct_sensor g_s2 = {{ 96.6, 92.6}};
#else
static struct_sensor g_s1 = {{-155.5,185.5}};
static struct_sensor g_s2 = {{185.5,-159.5}};
#endif

static struct_vec2d g_cor_1, g_cor_2;

static void COR_Scale(struct_vec2d *orig_ptr,
		      struct_vec2d_ushort *scaled_ptr,
		      double scale) {
  scaled_ptr->x = (_ushort)(scale * orig_ptr->x);
  scaled_ptr->y = (_ushort)(scale * orig_ptr->y);
}

static void COR_ScaleAccel(struct_vec2d *orig_ptr,
			   struct_vec2d_ushort *scaled_ptr) {
  COR_Scale(orig_ptr,scaled_ptr,Sample_GetScaleAccel());
}

static void COR_ScaleGlobal(struct_vec2d *orig_ptr,
			    struct_vec2d_ushort *scaled_ptr) {
  COR_Scale(orig_ptr,scaled_ptr,Sample_GetScaleGlobal());
}

static void COR_ScreenCoords(struct_vec2d *orig_ptr,
			     struct_vec2d_ushort *screen_ptr) {
  struct_vec2d_ushort scaled;
  COR_ScaleGlobal(orig_ptr, &scaled);
  screen_ptr->x = (gg_cvs_cor->w>>1)+scaled.x;
  screen_ptr->y = (gg_cvs_cor->h>>1)-scaled.y;
}

static void COR_DrawRectangle(struct_vec2d *corner_ptr,
			      struct_vec2d *span_ptr,
			      FL_COLOR color) {
  struct_vec2d_ushort ucorner,uspan;
  COR_ScreenCoords(corner_ptr, &ucorner);
  COR_ScaleGlobal(span_ptr, &uspan);
  fl_rect(ucorner.x,ucorner.y,uspan.x,uspan.y,color);
}

static void COR_DrawDisk(struct_vec2d *ctr_ptr, _ushort rad,
			 FL_COLOR color) {
  struct_vec2d_ushort ctr;
  COR_ScreenCoords(ctr_ptr, &ctr);
  fl_ovalf(ctr.x-rad, ctr.y-rad,rad+rad, rad+rad, color);
}

static void COR_Clear(void) {
  fl_winset(FL_ObjWin(gg_cvs_cor));
  fl_rectf(0,0,gg_cvs_cor->w,gg_cvs_cor->h,FL_BLACK);
}

static void COR_DrawAccelAndPerp(struct_vec2d_ushort *ctr_ptr,
				 struct_vec2d_ushort *acc_ptr,
				 FL_COLOR color) {
  fl_line(ctr_ptr->x,ctr_ptr->y,
	  ctr_ptr->x+acc_ptr->x,ctr_ptr->y-acc_ptr->y,
	  color);
  fl_linestyle(FL_DOT);
  fl_line(ctr_ptr->x,ctr_ptr->y,
	  ctr_ptr->x+acc_ptr->y,ctr_ptr->y+acc_ptr->x,
	  color);
  fl_line(ctr_ptr->x,ctr_ptr->y,
	  ctr_ptr->x-acc_ptr->y,ctr_ptr->y-acc_ptr->x,
	  color);
  fl_linestyle(FL_SOLID);
}

static void COR_DrawIt(_ushort rad, FL_COLOR color_1, FL_COLOR color_2) {
  COR_DrawDisk(&g_cor_1, rad, color_1);
  COR_DrawDisk(&g_cor_2, rad, color_2);
}

static void COR_DrawSensor(struct_sensor *s_ptr, _ushort rad,
			   FL_COLOR color,
			   FL_COLOR color_1, FL_COLOR color_2) {
  struct_vec2d_ushort ctr, scaled;
  COR_DrawDisk(&(s_ptr->pos), rad, color);
  COR_ScreenCoords(&(s_ptr->pos), &ctr);
  COR_ScaleAccel(&(s_ptr->accel1), &scaled);
  COR_DrawAccelAndPerp(&ctr, &scaled, color_1);
  COR_ScaleAccel(&(s_ptr->accel2), &scaled);
  COR_DrawAccelAndPerp(&ctr, &scaled, color_2);
  fl_point(ctr.x, ctr.y, FL_WHITE);
}

static void COR_DrawTable(void) {
  static struct_vec2d table_corner={-TABLE_SIDE_OV2,TABLE_SIDE_OV2};
  static struct_vec2d table_span={TABLE_SIDE,TABLE_SIDE};
  COR_DrawRectangle(&table_corner, &table_span, FL_WHITE);
}

static void COR_JohnCanny(struct_vec2d *p_ptr,
			  struct_vec2d *v1_ptr, struct_vec2d *v2_ptr,
			  struct_vec2d *cor_ptr) {
  double r1 = hypot(p_ptr->x, p_ptr->y);
  double v_diff = hypot(v2_ptr->x - v1_ptr->x, v2_ptr->y - v1_ptr->y); 
  double cross =
    (v2_ptr->x - v1_ptr->x)*p_ptr->y -  (v2_ptr->y - v1_ptr->y)*p_ptr->x; 
  double factor = r1/v_diff;
  if (cross < 0) factor = - factor;
  cor_ptr->y =  factor*(v1_ptr->x + v2_ptr->x);
  cor_ptr->x = -factor*(v1_ptr->y + v2_ptr->y);
  Sample_PrintOmega(0.5/factor);
}

static void COR_Estimate(struct_vec2d *p1_ptr, struct_vec2d *v1_ptr,
			 struct_vec2d *p2_ptr, struct_vec2d *v2_ptr,
			 struct_vec2d *cor_ptr) {
#ifdef XXX
  Line_InterPerp(p1_ptr, v1_ptr, p2_ptr, v2_ptr, cor_ptr);
#else
  COR_JohnCanny(p1_ptr, v1_ptr, v2_ptr, cor_ptr);
#endif
}

static double Dot1(struct_fit *fit_ptr, struct_fit *ref_ptr) {
  return(fit_ptr->s1*ref_ptr->s1+fit_ptr->c1*ref_ptr->c1);
}

static double Dot2(struct_fit *fit_ptr, struct_fit *ref_ptr) {
  return(fit_ptr->s2*ref_ptr->s2+fit_ptr->c2*ref_ptr->c2);
}

#define SIGN(x) ((x)<0?1:0)

static _byte NegativeForce(struct_fit *fit_ptr) { /* even quadrant? */
  return(fit_ptr->sin_phi_m1_sqr_m2<0);
}

static void MakeAccel(struct_fit *fit_x_ptr, _byte x_neg,
		      struct_fit *fit_y_ptr, _byte y_neg,
		      struct_fit *ref_ptr,
		      struct_vec2d *accel1_ptr,
		      struct_vec2d *accel2_ptr) {
  accel1_ptr->x = (x_neg ? -fit_x_ptr->magn1 : fit_x_ptr->magn1);
  accel2_ptr->x = (x_neg ? -fit_x_ptr->magn2 : fit_x_ptr->magn2);
  /* if ( Dot1(fit_x_ptr, ref_ptr) < 0) accel1_ptr->x = -accel1_ptr->x; */
  /* if ( Dot2(fit_x_ptr, ref_ptr) < 0) accel2_ptr->x = -accel2_ptr->x; */
  if ( NegativeForce(fit_x_ptr) ) {
    accel1_ptr->x = -accel1_ptr->x;
    accel2_ptr->x = -accel2_ptr->x;
  }
  accel1_ptr->y = (y_neg ? -fit_y_ptr->magn1 : fit_y_ptr->magn1);
  accel2_ptr->y = (y_neg ? -fit_y_ptr->magn2 : fit_y_ptr->magn2);
  /* if ( Dot1(fit_y_ptr, ref_ptr) < 0) accel1_ptr->y = -accel1_ptr->y; */
  /* if ( Dot2(fit_y_ptr, ref_ptr) < 0) accel2_ptr->y = -accel2_ptr->y; */
  if ( NegativeForce(fit_y_ptr) ) {
    accel1_ptr->y = -accel1_ptr->y;
    accel2_ptr->y = -accel2_ptr->y;
  }
}

/* EXPORTED */

void COR_GetIt(void) {
#ifdef OLD_TABLE
  MakeAccel(gg_fit_coeff+2, 0, /*  x1 */
	    gg_fit_coeff+3, 1, /* -y1 */
	    gg_fit_coeff+2,    /* x1 is reference */
	    &(g_s1.accel1), &(g_s1.accel2));
  MakeAccel(gg_fit_coeff+0, 1, /* -x2 */
	    gg_fit_coeff+1, 0, /*  y2 */
	    gg_fit_coeff+2,    /* x1 is reference */
	    &(g_s2.accel1), &(g_s2.accel2));
#else
  MakeAccel(gg_fit_coeff+3, 0, /*  x1 */
	    gg_fit_coeff+2, 0, /*  y1 */
	    gg_fit_coeff+3,    /* x1 is reference */
	    &(g_s1.accel1), &(g_s1.accel2));
  MakeAccel(gg_fit_coeff+0, 0, /*  x2 */
	    gg_fit_coeff+1, 1, /* -y2 */
	    gg_fit_coeff+3,    /* x1 is reference */
	    &(g_s2.accel1), &(g_s2.accel2));
#endif
  COR_Estimate(&(g_s1.pos), &(g_s1.accel1),
	       &(g_s2.pos), &(g_s2.accel1),
	       &g_cor_1);
  COR_Estimate(&(g_s1.pos), &(g_s1.accel2),
	       &(g_s2.pos), &(g_s2.accel2),
	       &g_cor_2);
}

void COR_Draw(void) {
  /* clears background */
  COR_Clear();
  /* table */
  COR_DrawTable();
  /* sensors, accel vectors, perp */
  COR_DrawSensor(&g_s1, P_RAD, FL_GREEN, FL_BLUE, FL_YELLOW);
  COR_DrawSensor(&g_s2, P_RAD, FL_RED,   FL_BLUE, FL_YELLOW);
  /* cor */
  COR_DrawIt(P_RAD>>1, FL_BLUE, FL_YELLOW);
}
