#define _XOPEN_SOURCE 

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>

#include "forms.h"
#include "board_ui.h"
#include "board_proto.h"

static XImage *g_ximage;
static XPoint *g_xpoint = NULL;
static Drawable g_win;
static Pixmap g_pixmap;
static GC g_gc;
static _byte g_shm;
static XShmSegmentInfo g_shminfo;

static int ExposeHandler(FL_OBJECT *ob, Window win, 
			 int win_width, int win_height,
			 XEvent *xev, void *user_data) {
  Sample_Redraw();
  return(0);
}

/* EXPORTED */

void Cvs_SetColor(long color) {
  XSetForeground(fl_display, g_gc, color);
}

void Cvs_DrawBaseLine(long color) {
  Cvs_SetColor(color);
  XDrawLine(fl_display, g_pixmap, g_gc, 
	    0, gg_cvs->h>>1, gg_cvs->w, gg_cvs->h>>1);
}

void Cvs_Clear(void) {
  Cvs_SetColor(0);
  XFillRectangle(fl_display,  g_pixmap, g_gc,
		 0, 0, gg_cvs->w, gg_cvs->h);
}

void Cvs_Refresh(void) {
  XCopyArea(fl_display, g_pixmap, g_win,
	    g_gc, 0, 0, gg_cvs->w, gg_cvs->h, 0, 0);
  XSync(fl_display, False);
}

void Cvs_Plot(short *data_ptr, _byte polyline,
	      _byte shift_x) {
  _uint i, i_step = (1<<shift_x);
  int (*xdraw_fn)() = polyline ? XDrawLines : XDrawPoints;
  XPoint *xpoint_ptr = g_xpoint+0;
  for (i=0; i < gg_cvs->w; i+= i_step, xpoint_ptr++, data_ptr++) {
    xpoint_ptr->x = i;
    xpoint_ptr->y = *data_ptr;
  }
  xdraw_fn(fl_display, g_pixmap, g_gc, g_xpoint, gg_cvs->w>>shift_x,
	   CoordModeOrigin);
}

void Cvs_Destroy(void) {
  XFreePixmap(fl_display, g_pixmap);
  if (g_shm) {
    XShmDetach(fl_display, &g_shminfo);
    XDestroyImage(g_ximage);
    shmdt(g_shminfo.shmaddr);
    shmctl(g_shminfo.shmid, IPC_RMID, 0);
  }
}

void Cvs_Init(void) {
  _uint depth = fl_state[fl_get_vclass()].depth;
  g_win = FL_ObjWin(gg_cvs);
  g_gc = fl_state[fl_get_vclass()].gc[8];
  g_shm = XShmQueryExtension(fl_display);
  printf("depth=%d, shm: %s\n", depth, g_shm?"ok":"not ok");
  if (g_shm) {
    g_ximage = XShmCreateImage(fl_display,
			     fl_state[fl_get_vclass()].xvinfo->visual,
			     depth,ZPixmap,NULL,
			     &g_shminfo,gg_cvs->w,gg_cvs->h);
    g_shminfo.shmid = shmget(IPC_PRIVATE,
			     g_ximage->bytes_per_line*g_ximage->height,
			     IPC_CREAT | 0777);
    g_shminfo.shmaddr = shmat(g_shminfo.shmid, 0, 0);
    g_shminfo.readOnly = False;
    XShmAttach(fl_display, &g_shminfo);
    g_pixmap = XShmCreatePixmap(fl_display,g_win,g_shminfo.shmaddr,
				&g_shminfo,gg_cvs->w,gg_cvs->h,depth);
  } else
    g_pixmap = XCreatePixmap(fl_display, g_win,
			      gg_cvs->w, gg_cvs->h, depth);
  g_xpoint = (XPoint *)malloc(sizeof(XPoint)*gg_cvs->w);
  fl_add_canvas_handler(gg_cvs, Expose, ExposeHandler, NULL);
}

