#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "forms.h"
#include "board_proto.h"

static struct_fit *g_fit_array = NULL;
/* EXPORTED */ struct_fit gg_fit_coeff[4];

static _ushort g_cvs_w;

static double GetSinPhiM1SqrM2(double s1, double c1,
			       double s2, double c2) {
  return(2.0*s1*c1*c2 + s2*(s1*s1 - c1*c1));
}

void Fit_Prod(struct_sample4 *sample4_ptr) {
  _ushort i, j;
  double sample, w_inv = 2.0/(double)g_cvs_w; /* normalize */
  struct_fit *fit_ptr, *prod_ptr;
  for (i = 4, prod_ptr = gg_fit_coeff+0; i; i--, prod_ptr++) {
    prod_ptr->s1 = 0.0; prod_ptr->c1 = 0.0;
    prod_ptr->s2 = 0.0; prod_ptr->c2 = 0.0;
  }
  for (i = g_cvs_w, fit_ptr = g_fit_array+0; i;
       i--, fit_ptr++, sample4_ptr++)
    for (j = 0, prod_ptr = gg_fit_coeff+0; j < 4; j++, prod_ptr++) {
      sample = (double)(sample4_ptr->sample[j]);
      prod_ptr->s1 += sample*fit_ptr->s1;
      prod_ptr->s2 += sample*fit_ptr->s2;
      prod_ptr->c1 += sample*fit_ptr->c1;
      prod_ptr->c2 += sample*fit_ptr->c2;
    }
  for (i = 4, prod_ptr = gg_fit_coeff+0; i; i--, prod_ptr++) {
    prod_ptr->s1 *= w_inv;
    prod_ptr->c1 *= w_inv;
    prod_ptr->magn1  = hypot(prod_ptr->s1, prod_ptr->c1);
    prod_ptr->phase1 = atan2(prod_ptr->s1, prod_ptr->c1);

    prod_ptr->s2 *= w_inv;
    prod_ptr->c2 *= w_inv;
    prod_ptr->magn2  = hypot(prod_ptr->s2, prod_ptr->c2);
    prod_ptr->phase2 = atan2(prod_ptr->s2, prod_ptr->c2);

    prod_ptr->sin_phi_m1_sqr_m2 = GetSinPhiM1SqrM2(prod_ptr->s1,prod_ptr->c1,
						   prod_ptr->s2,prod_ptr->c2);
  }
}

void Fit_Apply(_uint entry_num, _uint coeff_num,
	       double *fit_ptr, double *integr_ptr) {
  struct_fit *fn_ptr = g_fit_array+entry_num;
  struct_fit *prod_ptr = gg_fit_coeff+coeff_num;
  *fit_ptr =
    fn_ptr->s1 * prod_ptr->s1 +
    fn_ptr->c1 * prod_ptr->c1 +
    fn_ptr->s2 * prod_ptr->s2 +
    fn_ptr->c2 * prod_ptr->c2;
  *integr_ptr =
    fn_ptr->s1 * prod_ptr->c1 +
    fn_ptr->c1 * (-prod_ptr->s1) +
    fn_ptr->s2 * 0.5*prod_ptr->c2 +
    fn_ptr->c2 * (-0.5*prod_ptr->s2);
}

static double g_spcotp = 0.0;

double Fit_SamplesPerCycleOverTwoPi(void) {
  return(g_spcotp);
}

void Fit_MakeTable(_byte freq) {
  _ushort i;
  double th, d_th;
  struct_fit *fit_ptr;
  double spc = (double)(CYCLE_SAMPLES<<8) / (1.0+(double)freq);
  g_spcotp =  spc / TWO_PI;
  g_cvs_w = gg_cvs->w - ( gg_cvs->w % (_ushort)spc ); /* to avoid aliasing */
  d_th = 1.0/g_spcotp;
  for (i = gg_cvs->w, fit_ptr = g_fit_array, th = 0.0; i;
       i--, fit_ptr++, th += d_th) {
    fit_ptr->s1 = sin(th);
    fit_ptr->c1 = cos(th);
    fit_ptr->s2 = sin(th+th);
    fit_ptr->c2 = cos(th+th);
  }
}

void Fit_Init(void) {
  _byte freq = Globals_GetFreq();
  g_fit_array = (struct_fit *)malloc(gg_cvs->w*sizeof(struct_fit));
  Fit_MakeTable(freq);
  Board_SetFreq(freq);
}

