#include <stdio.h>
#include <stdlib.h>
#include "forms.h"
#include "board_ui.h"
#include "board_proto.h"

static FD_FormGlobals *g_globals_ui;

static double FreqActual(_byte freq) {
  static double factor = ((double)ISR_FREQ)/(256.0*(double)CYCLE_SAMPLES);
  return( (1.0+(double)freq)*factor );
}

static void Globals2UI(struct_glob *glob_ptr, _byte pic_num) {
  char buf[32];
  sprintf(buf, "%d", glob_ptr->stamp);
    fl_set_object_label(g_globals_ui->TextStamp[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_1);
    fl_set_input(g_globals_ui->InputStep1[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_2);
    fl_set_input(g_globals_ui->InputStep2[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_1-glob_ptr->step_2);
    fl_set_object_label(g_globals_ui->TextStepDiff[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_1+glob_ptr->step_2);
    fl_set_object_label(g_globals_ui->TextStepSum[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->phase);
    fl_set_input(g_globals_ui->InputPhase[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->mult1);
    fl_set_input(g_globals_ui->InputMult1[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->mult2);
    fl_set_input(g_globals_ui->InputMult2[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->shift);
    fl_set_input(g_globals_ui->InputShift, buf);
  sprintf(buf, "%d", glob_ptr->attack);
    fl_set_input(g_globals_ui->InputAttack, buf);
  sprintf(buf, "%d", glob_ptr->sustain);
    fl_set_input(g_globals_ui->InputSustain, buf);
  fl_set_choice(g_globals_ui->ChoiceShiftMode, (int)(glob_ptr->shift_mode+1));
  fl_set_choice(g_globals_ui->ChoiceDeb1[pic_num], (int)(glob_ptr->deb_1+1));
  fl_set_choice(g_globals_ui->ChoiceDeb2[pic_num], (int)(glob_ptr->deb_2+1));
  fl_set_choice(g_globals_ui->ChoiceOnMode[pic_num],
		(int)(glob_ptr->on_mode+1));
  fl_set_counter_value(g_globals_ui->CounterFreq, (double)(glob_ptr->freq));
  sprintf(buf, "%.2f Hz", FreqActual(glob_ptr->freq));
  fl_set_object_label(g_globals_ui->TextFreq, buf);
}


static void UI2Struct(_byte pic_num, struct_glob *glob_ptr) {
  glob_ptr->stamp = 0;
  /* wave */
  glob_ptr->phase =
    (_byte)atoi(fl_get_input(g_globals_ui->InputPhase[pic_num]));
  glob_ptr->mult1 =
    (_byte)atoi(fl_get_input(g_globals_ui->InputMult1[pic_num]));
  glob_ptr->mult2 =
    (_byte)atoi(fl_get_input(g_globals_ui->InputMult2[pic_num]));
  /* envelope */
  glob_ptr->attack  =
    (_ushort)atoi(fl_get_input(g_globals_ui->InputAttack));
  glob_ptr->sustain =
    (_ushort)atoi(fl_get_input(g_globals_ui->InputSustain));
  glob_ptr->shift =
    (_byte)atoi(fl_get_input(g_globals_ui->InputShift));
  glob_ptr->shift_mode =
    (_byte)(fl_get_choice(g_globals_ui->ChoiceShiftMode)-1);
  /* debug */
  glob_ptr->deb_1 =
    (_byte)(fl_get_choice(g_globals_ui->ChoiceDeb1[pic_num])-1);
  glob_ptr->deb_2 =
    (_byte)(fl_get_choice(g_globals_ui->ChoiceDeb2[pic_num])-1);
  /* step */
  glob_ptr->step_1 =
    (short)atoi(fl_get_input(g_globals_ui->InputStep1[pic_num]));
  glob_ptr->step_2 =
    (short)atoi(fl_get_input(g_globals_ui->InputStep2[pic_num]));
  /* on_mode */
  glob_ptr->on_mode =
    (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceOnMode[pic_num])-1);
  /* freq */
  glob_ptr->freq =
    (_byte)fl_get_counter_value(g_globals_ui->CounterFreq);
}

static void FillOutGlobals(void) {
  struct_glob glob_a, glob_b;
  Board_GetGlob(&glob_a, &glob_b);
  Globals2UI(&glob_a, 0);
  Globals2UI(&glob_b, 1);
}

static void Refresh(void) {
  FillOutGlobals();
}

static void Glob2Structs(struct_glob *glob_ptr,
			 struct_wave *wave_ptr,
			 struct_envel *envel_ptr,
			 enum_DEB deb[2],
			 short step[2],
			 enum_ON_MODE *on_mode_ptr,
			 _byte *freq_ptr) {
  wave_ptr->phase = glob_ptr->phase;
  wave_ptr->mult1 = glob_ptr->mult1;
  wave_ptr->mult2 = glob_ptr->mult2;

  envel_ptr->attack = glob_ptr->attack;
  envel_ptr->sustain = glob_ptr->sustain;
  envel_ptr->shift = glob_ptr->shift;
  envel_ptr->shift_mode = glob_ptr->shift_mode;

  deb[0] = glob_ptr->deb_1;
  deb[1] = glob_ptr->deb_2;

  step[0] = glob_ptr->step_1;
  step[1] = glob_ptr->step_2;

  *on_mode_ptr = glob_ptr->on_mode;

  *freq_ptr = glob_ptr->freq;
}

static void Struct2Board(struct_glob *glob_a_ptr,
			 struct_glob *glob_b_ptr) {
  struct_wave wave_a, wave_b;
  struct_envel envel;
  enum_DEB deb[4];
  short step[4];
  _byte freq;
  enum_ON_MODE mode_a, mode_b;
  Glob2Structs(glob_a_ptr, &wave_a, &envel, deb+0, step+0,
	       &mode_a, &freq);
  Glob2Structs(glob_b_ptr, &wave_b, &envel, deb+2, step+2,
	       &mode_b, &freq);
  Board_SetWave(&wave_a, &wave_b);
  Board_SetEnvelope(&envel, &envel);
  Board_SetDebug(deb);
  Board_SetSteps(step);
  Board_SetOnMode(mode_a, mode_b);
  Fit_MakeTable(freq);
  Board_SetFreq(freq);
}

static void Struct2File(FILE *fp, struct_glob *glob_ptr) {
  if (1 != fwrite((void *)glob_ptr, sizeof(struct_glob), 1, fp)) 
    fprintf(stderr, "ERROR: Struct2File\n");
}

static void File2Struct(FILE *fp, struct_glob *glob_ptr) {
  if (1 != fread((void *)glob_ptr, sizeof(struct_glob), 1, fp))
    fprintf(stderr, "ERROR: File2Struct\n");
}

static void Glob2Step(struct_glob *glob_a_ptr, struct_glob *glob_b_ptr,
		      short step[4]) {
  step[0] = glob_a_ptr->step_1;
  step[1] = glob_a_ptr->step_2; 
  step[2] = glob_b_ptr->step_1;
  step[3] = glob_b_ptr->step_2;
} 

static void OffSetStepsOn(short step[4]) {
  Board_Off();
  Board_SetSteps(step);
  Board_On();
}

/* BEGIN: CALLBACKS */

#define STEP_INC 4

void CbkBtnTandemA(FL_OBJECT *ob, long data) {
  short step[4];
  struct_glob glob_a, glob_b;
  short delta = (short)(STEP_INC*data);
  Board_GetGlob(&glob_a, &glob_b);
  glob_a.step_1 += delta;
  glob_a.step_2 -= delta;
  Globals2UI(&glob_a, 0);
  Glob2Step(&glob_a, &glob_b, step);
  OffSetStepsOn(step);
}

void CbkBtnOppositeA(FL_OBJECT *ob, long data) {
  short step[4];
  struct_glob glob_a, glob_b;
  short delta = (short)(STEP_INC*data);
  Board_GetGlob(&glob_a, &glob_b);
  glob_a.step_1 += delta;
  glob_a.step_2 += delta;
  Globals2UI(&glob_a, 0);
  Glob2Step(&glob_a, &glob_b, step);
  OffSetStepsOn(step);
}

void CbkBtnTandemB(FL_OBJECT *ob, long data) {
  short step[4];
  struct_glob glob_a, glob_b;
  short delta = (short)(STEP_INC*data);
  Board_GetGlob(&glob_a, &glob_b);
  glob_b.step_1 += delta;
  glob_b.step_2 -= delta;
  Globals2UI(&glob_b, 1);
  Glob2Step(&glob_a, &glob_b, step);
  OffSetStepsOn(step);
}

void CbkBtnOppositeB(FL_OBJECT *ob, long data) {
  short step[4];
  struct_glob glob_a, glob_b;
  short delta = (short)(STEP_INC*data);
  Board_GetGlob(&glob_a, &glob_b);
  glob_b.step_1 += delta;
  glob_b.step_2 += delta;
  Globals2UI(&glob_b, 1);
  Glob2Step(&glob_a, &glob_b, step);
  OffSetStepsOn(step);
}

void CbkBtnSave(FL_OBJECT *ob, long data) {
  const char *fname =
    fl_show_fselector("Save to:", ".", "*.sav", "globals.sav");
  if (fname) {
    FILE *fp = fopen(fname, "w");
    struct_glob glob_a, glob_b;
    UI2Struct(0, &glob_a);
    UI2Struct(1, &glob_b);
    Struct2File(fp, &glob_a);
    Struct2File(fp, &glob_b);
    fclose(fp);
    fprintf(stderr, "saved to '%s'\n", fname);
  }
}

void CbkBtnLoad(FL_OBJECT *ob, long data) {
  const char *fname
    = fl_show_fselector("Load from:", ".", "*.sav", "globals.sav");
  FILE *fp;
  if (fname) {
    if ((fp = fopen(fname, "r"))) {
      struct_glob glob_a, glob_b;
      File2Struct(fp, &glob_a);
      File2Struct(fp, &glob_b);
      fclose(fp);
      Struct2Board(&glob_a, &glob_b);
      Refresh(); /* for debugging */
      fprintf(stderr, "loaded from '%s'\n", fname);
    }  else
      fprintf(stderr, "file '%s' not found\n", fname);
  }
}

void CbkUI2Globals(FL_OBJECT *ob, long data) {
  struct_glob glob_a, glob_b;
  UI2Struct(0, &glob_a);
  UI2Struct(1, &glob_b);
  Struct2Board(&glob_a, &glob_b);
  Refresh(); /* for debugging */
}

void CbkBtnOn(FL_OBJECT *ob, long data) {
  if (fl_get_button(ob))
    Board_On();
  else
    Board_Off();
}

void CbkBtnRefresh(FL_OBJECT *ob, long data) {
  Refresh();
}

void CbkBtnReset(FL_OBJECT *ob, long data) {
  Board_Reset();
  Refresh();
}

static void InitChoice(FL_OBJECT *choice_ptr, char *entries[]) {
  fl_clear_choice(choice_ptr);
  for ( ; *entries; entries++)
    fl_addto_choice(choice_ptr, *entries);
}

static void InitInputs(void) {
  _byte i;
  for (i = 0; i < 2; i++) {
    fl_set_input_return(g_globals_ui->InputPhase[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputMult1[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputMult2[i],   FL_RETURN_END);
  }
  fl_set_input_return(g_globals_ui->InputShift,   FL_RETURN_END);
  fl_set_input_return(g_globals_ui->InputAttack,  FL_RETURN_END);
  fl_set_input_return(g_globals_ui->InputSustain, FL_RETURN_END);
}

void Globals_Init(void) {
  static char *entries_deb[] = {"wave","envel","signl",NULL};
  static char *entries_on_mode[] = {"sust","pulse","train",NULL};
  static char *entries_shift_mode[] = {"up_mid","dn_mid",
				       "up_top","dn_top",NULL};
  int i;
  char info_a[64], info_b[64];
  g_globals_ui = create_form_FormGlobals();
  /* fill-in form initialization code */
  Board_GetVersionInfo(info_a, info_b);
  fl_set_object_label(g_globals_ui->TextVersionInfo, info_a);
  InitInputs();
  for (i = 0; i < 2; i++) {
    InitChoice(g_globals_ui->ChoiceDeb1[i], entries_deb);
    InitChoice(g_globals_ui->ChoiceDeb2[i], entries_deb);
    InitChoice(g_globals_ui->ChoiceOnMode[i], entries_on_mode);
  }
  InitChoice(g_globals_ui->ChoiceShiftMode, entries_shift_mode);
  FillOutGlobals();
  /* show the first form */
  fl_show_form(g_globals_ui->FormGlobals,
	       FL_PLACE_SIZE,FL_TRANSIENT,"PIC Globals");
}

_byte Globals_GetFreq(void) {
  return((_byte)fl_get_counter_value(g_globals_ui->CounterFreq));
}
