#include <stdio.h>
#include <math.h>
#include "board_proto.h"

#define VERY_SMALL 1.0e-9

double Line_Inv(double val) {
  if (fabs(val) < VERY_SMALL) {
    fprintf(stderr, "ERROR: Line_Inv()\n");
    val = VERY_SMALL;
  }
  return(1.0/val);
}

void Line_Scale(struct_vec2d *p_ptr, double scale,
		struct_vec2d *p_scaled_ptr) {
  p_scaled_ptr->x = scale * p_ptr->x; 
  p_scaled_ptr->y = scale * p_ptr->y; 
}

void Line_Sum(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr,
	      struct_vec2d *p1_plus_p2_ptr) {
  p1_plus_p2_ptr->x = p1_ptr->x + p2_ptr->x;
  p1_plus_p2_ptr->y = p1_ptr->y + p2_ptr->y;
}

void Line_Difference(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr,
		     struct_vec2d *p1_minus_p2_ptr) {
  p1_minus_p2_ptr->x = p1_ptr->x - p2_ptr->x;
  p1_minus_p2_ptr->y = p1_ptr->y - p2_ptr->y;
}

double Line_DotProduct(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr) {
  return(p1_ptr->x * p2_ptr->x + p1_ptr->y * p2_ptr->y);
}

double Line_CrossProduct(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr) {
  return(p1_ptr->x * p2_ptr->y - p1_ptr->y * p2_ptr->x);
}

void Line_InterPerp(struct_vec2d *p1_ptr, struct_vec2d *v1_ptr,
		    struct_vec2d *p2_ptr, struct_vec2d *v2_ptr,
		    struct_vec2d *inter_ptr) {
  double det_inv = Line_Inv( Line_CrossProduct(v1_ptr, v2_ptr) );
  double det_x, det_y;
  struct_vec2d c, v;
  c.x = Line_DotProduct(v1_ptr, p1_ptr);
  c.y = Line_DotProduct(v2_ptr, p2_ptr);
  v.x = v1_ptr->y; v.y = v2_ptr->y;
  det_x = Line_CrossProduct(&c, &v);
  v.x = v1_ptr->x; v.y = v2_ptr->x;
  det_y = Line_CrossProduct(&v, &c);
  inter_ptr->x = det_x * det_inv;
  inter_ptr->y = det_y * det_inv;
}
