#include "board.h" 

#define M_PI 3.14159265358979323846
#define TWO_PI (M_PI+M_PI)

extern double hypot(double x, double y);
 
typedef struct {
  double s1, c1, s2, c2;
  double sin_phi_m1_sqr_m2;
  /* UNSTABLE MAPS: need sqrt() and atan() */
  double magn1, magn2;
  double phase1, phase2;
} struct_fit;

extern struct_fit gg_fit_coeff[4];

typedef struct {
  double x, y;
} struct_vec2d;

#ifdef FL_FORMS_H
extern FL_OBJECT *gg_cvs, *gg_cvs_cor;
#endif

/* PARALLEL */

extern   int Parallel_Init(void);
extern  void Parallel_Write_Data(_byte b);
extern  void Parallel_Write_Status(_byte b);
extern  void Parallel_Write_Control(_byte b);
extern _byte Parallel_Read_Data(void);
extern _byte Parallel_Read_Status(void);
extern _byte Parallel_Read_Control(void);

/* LINE */

extern double Line_Inv(double val);
extern void Line_Scale(struct_vec2d *p_ptr, double scale,
		       struct_vec2d *p_scaled_ptr);
extern void Line_Sum(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr,
		     struct_vec2d *p1_plus_p2_ptr);
extern void Line_Difference(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr,
			    struct_vec2d *p1_minus_p2_ptr);
extern double Line_DotProduct(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr);
extern double Line_CrossProduct(struct_vec2d *p1_ptr, struct_vec2d *p2_ptr);
extern void Line_InterPerp(struct_vec2d *p1_ptr, struct_vec2d *v1_ptr,
			   struct_vec2d *p2_ptr, struct_vec2d *v2_ptr,
			   struct_vec2d *inter_ptr);

/* FIT */

extern   void Fit_Init(void);
extern   void Fit_Apply(_uint entry_num, _uint coeff_num,
			double *fit_ptr, double *integr_ptr);
extern   void Fit_Prod(struct_sample4 *sample4_ptr);
extern   void Fit_MakeTable(_byte freq);
extern double Fit_SamplesPerCycleOverTwoPi(void);

/* CVS */

extern void Cvs_DrawBaseLine(long color); /* g_pixmap */
extern void Cvs_Refresh(void); /* g_pixmap */
extern void Cvs_Plot(short *data_ptr, _byte polyline, _byte shift_x);
extern void Cvs_Init(void);
extern void Cvs_Destroy(void);

extern void Cvs_SetColor(long color); /* both */
extern void Cvs_Clear(void); /* both */

/* SAMPLE */

extern   void Sample_Redraw(void);
extern   void Sample_Init(void);
extern double Sample_GetScaleAccel(void);
extern double Sample_GetScaleGlobal(void);
extern   void Sample_PrintOmega(double omega);

/* GLOBALS */

extern  void Globals_Init(void);
extern _byte Globals_GetFreq(void);

/* COR */

extern void COR_GetIt(void);
extern void COR_Draw(void);
