#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "forms.h"
#include "board_ui.h"
#include "board_proto.h"

/* EXPORTED */ FL_OBJECT *gg_cvs, *gg_cvs_cor;

static int g_avg[4]={0,0,0,0};
static struct_sample4 *g_sample4_array;
static struct_sample4 *g_fit4_array;
static struct_sample4 *g_integr4_array;
static _byte *g_valid_array;
static short *g_y_array;
static _byte  g_shift_x, g_shift_y;
static _uint  g_trigger_offset=0;

static FD_FormSample *g_sample_ui;

#define BLACK   0x000000
#define RED     0xff0000
#define RED_B   0xff4040
#define GREEN   0x00ff00
#define BLUE    0x0000ff
#define YELLOW  0xffff00
#define MAGENTA 0xff00ff
#define CYAN    0x00ffff

static void FitText_Init(void) {
  _byte i;
  char buf[8];
  for (i = 0; i < 4; i++) {
    sprintf(buf,"c1_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleC1[i],    buf);
    sprintf(buf,"s1_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleS1[i],    buf);
    sprintf(buf,"m1_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleMagn1[i], buf);

    sprintf(buf,"s2_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleC2[i],    buf);
    sprintf(buf,"c2_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleS2[i],    buf);
    sprintf(buf,"m2_%d", i);
    fl_set_object_label(g_sample_ui->TextSampleMagn2[i], buf);
  }
}

static void FitText_Refresh(void) {
  _byte i;
  char buf[8];
  if (fl_get_button(g_sample_ui->BtnTextSample1)) {
    for (i = 0; i < 4; i++) {
      sprintf(buf,"%.0f", gg_fit_coeff[i].c1);
      fl_set_object_label(g_sample_ui->TextSampleC1[i],    buf);
      sprintf(buf,"%.0f", gg_fit_coeff[i].s1);
      fl_set_object_label(g_sample_ui->TextSampleS1[i],    buf);
      sprintf(buf,"%.0f", gg_fit_coeff[i].magn1);
      fl_set_object_label(g_sample_ui->TextSampleMagn1[i], buf);
    }
  }
  if (fl_get_button(g_sample_ui->BtnTextSample2)) {
    for (i = 0; i < 4; i++) {
      sprintf(buf,"%.0f", gg_fit_coeff[i].c2);
      fl_set_object_label(g_sample_ui->TextSampleC2[i],    buf);
      sprintf(buf,"%.0f", gg_fit_coeff[i].s2);
      fl_set_object_label(g_sample_ui->TextSampleS2[i],    buf);
      sprintf(buf,"%.0f", gg_fit_coeff[i].magn2);
      fl_set_object_label(g_sample_ui->TextSampleMagn2[i], buf);
    }
  }
  for (i = 0; i < 4; i++)
    fl_set_object_label(g_sample_ui->TextSampleSGN[i],
			gg_fit_coeff[i].sin_phi_m1_sqr_m2>0 ? "+" : "-");
}

static void GetStats(int n, double avg[4], double std_dev[4]) {
  int i, j;
  double sample, sum_x[4], sum_x2[4];
  double n_inv = 1.0/(double)n;
  struct_sample4 *sample4_array =
    (struct_sample4 *)malloc(n*sizeof(struct_sample4));
  Board_GetSamples(n, sample4_array);
  for(i=0; i<4; i++) {
    sum_x[i] = 0.0;
    sum_x2[i] = 0.0;
  }
  for (i=0; i<n; i++)
    for (j=0; j<4; j++) {
      sample = (double)(sample4_array[i].sample[j]);
      sum_x[j] += sample;
      sum_x2[j] += sample*sample;
    }
  free(sample4_array);
  for (i=0; i<4; i++) {
    avg[i] = n_inv*sum_x[i];
    std_dev[i] = sqrt(n_inv*sum_x2[i] - avg[i]*avg[i]);
  }
}

static void Sample_SubtractAverage(void) {
  _uint i, j;
  struct_sample4 *sample4_ptr = g_sample4_array+0;
  for (i = gg_cvs->w; i; i--, sample4_ptr++)
    for (j = 0; j < 4; j++)
      sample4_ptr->sample[j] -= g_avg[j];
}

static void Sample_MakeContiguous(void) {
  _uint i, j, i_cont, base = g_sample4_array[0].stamp;
  struct_sample4 *sample4_ptr, *last4_ptr;
  short *dest_ptr;
  _byte *valid_ptr;
  /* zero valid field */
  memset((void *)g_valid_array, 0, gg_cvs->w);
  /* eliminate out-of-bounds samples */
  for (i = gg_cvs->w, sample4_ptr = g_sample4_array+gg_cvs->w-1;
       i; i--, sample4_ptr--) {
    i_cont = sample4_ptr->stamp - base;
    if (i_cont < gg_cvs->w) {
      g_valid_array[i_cont] = 1;
      for (j = 0, dest_ptr = g_sample4_array[i_cont].sample+0; j < 4;
	   j++, dest_ptr++)
	*dest_ptr = sample4_ptr->sample[j];
    }
  }
  /* fill out missing samples */
  for (i = 0, sample4_ptr = last4_ptr = g_sample4_array+0,
	 valid_ptr = g_valid_array+0;
       i < gg_cvs->w; i++, sample4_ptr++, valid_ptr++)
    if (*valid_ptr)
      last4_ptr = sample4_ptr;
    else
      for (j = 0, dest_ptr = sample4_ptr->sample+0; j<4; j++, dest_ptr++)
	*dest_ptr = last4_ptr->sample[j];
}

static void Sample_GetFit(void) {
  _uint i, j;
  double fit, integr;
  Fit_Prod(g_sample4_array);
  for (i = 0; i < gg_cvs->w; i++) {
    g_fit4_array[i].stamp = g_integr4_array[i].stamp = i;
    for (j = 0; j < 4; j++) {
      Fit_Apply(i, j, &fit, &integr);
      g_fit4_array[i].sample[j]    = (short)fit;
      g_integr4_array[i].sample[j] = (short)integr;
    }
  }
}

static void Sample_GetTrigger(void) {
  if (fl_get_button(g_sample_ui->BtnTriggerOn)) {
    _byte i;
    double angle;
    for (i = 0; !fl_get_button(g_sample_ui->BtnTrigger[i]); i++)
      ;
    angle = gg_fit_coeff[i].phase1;
    if (angle < 0) angle += TWO_PI;
    g_trigger_offset = (_uint)(angle*Fit_SamplesPerCycleOverTwoPi());
  } else
    g_trigger_offset = 0;
}

static void Sample_Refresh(void) {
  Board_GetSamples(gg_cvs->w, g_sample4_array);
  Sample_MakeContiguous();
  Sample_SubtractAverage();
  Sample_GetFit();
  Sample_GetTrigger();
  COR_GetIt();
  Sample_Redraw();
}

static int IdleCallback(/* XEvent */ void *ev, void *data) {
  if (fl_get_button(g_sample_ui->BtnGraphContinuous))
    Sample_Refresh();
  return(1);
}

static short SignedShift(int val, _byte shift) {
  return(val < 0 ? -((-val)>>shift) : val>>shift);
}

static void Sample_PlotY(struct_sample4 *sample4_ptr, _byte sample_num,
		    _byte polyline) {
  _uint i;
  short *y_ptr = g_y_array+0;
  struct_sample4 *trig4_ptr_max = sample4_ptr + gg_cvs->w - 1;
  struct_sample4 *trig4_ptr = sample4_ptr + g_trigger_offset;
  for (i = gg_cvs->w>>g_shift_x; i; i--, trig4_ptr++, y_ptr++) {
    if (trig4_ptr > trig4_ptr_max) trig4_ptr = sample4_ptr;
    *y_ptr =
      (gg_cvs->h>>1)-SignedShift(trig4_ptr->sample[sample_num],g_shift_y);
  }
  Cvs_Plot(g_y_array, polyline, g_shift_x);
}

static void Sample_Destroy(void) {
  free(g_valid_array);
  free(g_y_array);
  free(g_sample4_array);
  free(g_integr4_array);
  free(g_fit4_array);
}
		    
/* EXPORTED */

void Sample_Redraw(void) {
  static _uint g_caption_colors[4] = {CYAN,GREEN,YELLOW,MAGENTA};
  _byte i;
  Cvs_Clear();
  Cvs_DrawBaseLine(RED);
  for (i = 0; i < 4; i++) {
    Cvs_SetColor(g_caption_colors[i]);
    if (fl_get_button(g_sample_ui->BtnView[i]))
      Sample_PlotY(g_sample4_array, i,
		   fl_get_button(g_sample_ui->BtnSolidView));
    if (fl_get_button(g_sample_ui->BtnSineFit[i]))
      Sample_PlotY(g_fit4_array, i,
		   fl_get_button(g_sample_ui->BtnSolidFit));
    if (fl_get_button(g_sample_ui->BtnIntegrate[i]))
      Sample_PlotY(g_integr4_array, i,
		   fl_get_button(g_sample_ui->BtnSolidIntegr));
  }
  Cvs_Refresh();
  COR_Draw();
  FitText_Refresh();
}

void CbkRedraw(FL_OBJECT *ob, long data) {
  Sample_Redraw();
}

void CbkCounterShiftY(FL_OBJECT *ob, long data) {
  g_shift_y = (_byte)fl_get_counter_value(g_sample_ui->CounterShiftY);
  Sample_Redraw();
}

void CbkCounterShiftX(FL_OBJECT *ob, long data) {
  g_shift_x = (_byte)fl_get_counter_value(g_sample_ui->CounterShiftX);
  Sample_Redraw();
}

void CbkBtnQuit(FL_OBJECT *ob, long data) {
  Cvs_Destroy();
  Sample_Destroy();
  fl_finish();
  exit(1);
}

void CbkBtnGraph(FL_OBJECT *ob, long data) {
  Sample_Refresh();
}

void CbkBtnGetStats(FL_OBJECT *ob, long data) {
  int i;
  int n = atoi(fl_get_input(g_sample_ui->InputQty));
  if (n<1) {
    for (i=0; i<4; i++) {
      fl_set_object_label(g_sample_ui->TextSampleAvg[i], "XXX");
      fl_set_object_label(g_sample_ui->TextSampleStdDev[i], "XXX");
    }
  } else {
    char buf[32];
    double avg[4], std_dev[4];
    GetStats(n, avg, std_dev);
    for (i=0; i<4; i++) {
      sprintf(buf, "%.0f", avg[i]);
      fl_set_object_label(g_sample_ui->TextSampleAvg[i], buf);
      sprintf(buf, "%.0f", std_dev[i]);
      fl_set_object_label(g_sample_ui->TextSampleStdDev[i], buf);
    }
    for (i=0; i<4; i++)
      g_avg[i] = (int)avg[i];
  }
}

void Sample_Init(void) {
  char buf[16];
  _uint s4_size;
  g_sample_ui = create_form_FormSample();
  gg_cvs = g_sample_ui->CanvasAccel;
  gg_cvs_cor = g_sample_ui->CanvasCOR;
  g_shift_x = g_shift_y = 0;

  g_valid_array = (_byte *)malloc(gg_cvs->w);
  g_y_array = (_ushort *)malloc(gg_cvs->w*sizeof(short));
  s4_size = gg_cvs->w*sizeof(struct_sample4);
  g_sample4_array = (struct_sample4 *)malloc(s4_size);
  g_integr4_array = (struct_sample4 *)malloc(s4_size);
  g_fit4_array    = (struct_sample4 *)malloc(s4_size);

  sprintf(buf, "%d", gg_cvs->w<<1);
  fl_set_input(g_sample_ui->InputQty, buf);
  fl_set_input_return(g_sample_ui->InputQty, FL_RETURN_END);
  fl_set_counter_step(g_sample_ui->CounterShiftX, 1.0, 1.0);
  fl_set_counter_step(g_sample_ui->CounterShiftY, 1.0, 1.0);
  fl_set_slider_value(g_sample_ui->SliderScaleGlobal, 0.75);
  fl_show_form(g_sample_ui->FormSample,FL_PLACE_SIZE,FL_TRANSIENT,
	       "Sample Viewer");
  FitText_Init();
  CbkBtnGetStats(NULL, 0);
  Cvs_Init();
  Fit_Init();
  fl_set_idle_callback((FL_APPEVENT_CB)IdleCallback, NULL);
  Sample_Refresh();
}

double Sample_GetScaleAccel(void) {
  return(fl_get_slider_value(g_sample_ui->SliderScaleAccel));
}

double Sample_GetScaleGlobal(void) {
  return(fl_get_slider_value(g_sample_ui->SliderScaleGlobal));
}

void Sample_PrintOmega(double omega) {
  char buf[16];
  sprintf(buf,"%.2f", omega);
  fl_set_object_label(g_sample_ui->TextSampleOmega, buf);
}
