#include <stdio.h>
#include <stdlib.h>
#include "forms.h"
#include "board_ui.h"
#include "board.h"

static FD_FormGlobals *g_globals_ui;

static void Globals2UI(struct_glob *glob_ptr, _byte pic_num) {
  char buf[32];
  sprintf(buf, "%d", glob_ptr->stamp);
    fl_set_object_label(g_globals_ui->TextStamp[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_1);
    fl_set_input(g_globals_ui->InputStep1[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->step_2);
    fl_set_input(g_globals_ui->InputStep2[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->phase);
    fl_set_input(g_globals_ui->InputPhase[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->mult1);
    fl_set_input(g_globals_ui->InputMult1[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->mult2);
    fl_set_input(g_globals_ui->InputMult2[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->shift);
    fl_set_input(g_globals_ui->InputShift[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->attack);
    fl_set_input(g_globals_ui->InputAttack[pic_num], buf);
  sprintf(buf, "%d", glob_ptr->sustain);
    fl_set_input(g_globals_ui->InputSustain[pic_num], buf);
  fl_set_choice(g_globals_ui->ChoiceDeb1[pic_num], (int)(glob_ptr->deb_1+1));
  fl_set_choice(g_globals_ui->ChoiceDeb2[pic_num], (int)(glob_ptr->deb_2+1));
  fl_set_choice(g_globals_ui->ChoiceOnMode[pic_num],
		(int)(glob_ptr->on_mode+1));
}

static void UI2Wave(_byte pic_num, struct_wave *wave_ptr) {
  wave_ptr->phase =
    (_byte)atoi(fl_get_input(g_globals_ui->InputPhase[pic_num]));
  wave_ptr->mult1 =
    (_byte)atoi(fl_get_input(g_globals_ui->InputMult1[pic_num]));
  wave_ptr->mult2 =
    (_byte)atoi(fl_get_input(g_globals_ui->InputMult2[pic_num]));
}

static void UI2Envelope(_byte pic_num, struct_envelope *envel_ptr) {
  envel_ptr->attack  =
    (_byte)atoi(fl_get_input(g_globals_ui->InputAttack[pic_num]));
  envel_ptr->sustain =
    (_byte)atoi(fl_get_input(g_globals_ui->InputSustain[pic_num]));
  envel_ptr->shift =
    (_byte)atoi(fl_get_input(g_globals_ui->InputShift[pic_num]));
}

static void UI2Debug(enum_DEB deb[4]) {
  deb[0] = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceDeb1[0])-1);
  deb[1] = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceDeb2[0])-1);
  deb[2] = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceDeb1[1])-1);
  deb[3] = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceDeb2[1])-1);
}

static void UI2Steps(short step[4]) {
  step[0] = (short)atoi(fl_get_input(g_globals_ui->InputStep1[0]));
  step[1] = (short)atoi(fl_get_input(g_globals_ui->InputStep2[0]));
  step[2] = (short)atoi(fl_get_input(g_globals_ui->InputStep1[1]));
  step[3] = (short)atoi(fl_get_input(g_globals_ui->InputStep2[1]));
}

static void UI2OnMode(enum_ON_MODE *mode_a_ptr, enum_ON_MODE *mode_b_ptr) {
  *mode_a_ptr = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceOnMode[0])-1);
  *mode_b_ptr = (enum_DEB)(fl_get_choice(g_globals_ui->ChoiceOnMode[1])-1);
}

static void FillOutGlobals(void) {
  struct_glob glob_a, glob_b;
  Board_GetGlob(&glob_a, &glob_b);
  Globals2UI(&glob_a, 0);
  Globals2UI(&glob_b, 1);
}

static void Refresh(void) {
  FillOutGlobals();
}

/* BEGIN: CALLBACKS */

void CbkUI2Globals(FL_OBJECT *ob, long data) {
  struct_wave wave_a, wave_b;
  struct_envelope envel_a, envel_b;
  enum_DEB deb[4];
  short step[4];
  enum_ON_MODE mode_a, mode_b;
  UI2Wave(0, &wave_a);
  UI2Wave(1, &wave_b);
  UI2Envelope(0, &envel_a);
  UI2Envelope(1, &envel_b);
  UI2Debug(deb);
  UI2Steps(step);
  UI2OnMode(&mode_a, &mode_b);
  Board_Off();
  fl_set_button(g_globals_ui->BtnOn, 0);
  Board_SetWave(&wave_a, &wave_b);
  Board_SetEnvelope(&envel_a, &envel_b);
  Board_SetDebug(deb);
  Board_SetSteps(step);
  Board_SetOnMode(mode_a, mode_b);
  Refresh(); /* for debugging */
}

void CbkBtnOn(FL_OBJECT *ob, long data) {
  if (fl_get_button(ob))
    Board_On();
  else
    Board_Off();
}

void CbkBtnRefresh(FL_OBJECT *ob, long data) {
  Refresh();
}

void CbkBtnReset(FL_OBJECT *ob, long data) {
  Board_Reset();
  Refresh();
}

void CbkBtnQuit(FL_OBJECT *ob, long data) {
  fl_finish();
  exit(1);
}

static void InitChoiceDeb(FL_OBJECT *choice_ptr) {
  fl_clear_choice(choice_ptr);
  fl_addto_choice(choice_ptr, "wave");
  fl_addto_choice(choice_ptr, "envel");
  fl_addto_choice(choice_ptr, "signl");
}

static void InitChoiceOnMode(FL_OBJECT *choice_ptr) {
  fl_clear_choice(choice_ptr);
  fl_addto_choice(choice_ptr, "sust");
  fl_addto_choice(choice_ptr, "pulse");
  fl_addto_choice(choice_ptr, "train");
}

static void InitInputs(void) {
  _byte i;
  for (i = 0; i < 2; i++) {
    fl_set_input_return(g_globals_ui->InputPhase[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputMult1[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputMult2[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputShift[i],   FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputAttack[i],  FL_RETURN_END);
    fl_set_input_return(g_globals_ui->InputSustain[i], FL_RETURN_END);
  }
}

void FormGlobals_Init(void) {
  int i;
  char info_a[64], info_b[64];
  g_globals_ui = create_form_FormGlobals();
  /* fill-in form initialization code */
  Board_GetVersionInfo(info_a, info_b);
  fl_set_object_label(g_globals_ui->TextVersionInfo, info_a);
  InitInputs();
  for (i = 0; i < 2; i++) {
    InitChoiceDeb(g_globals_ui->ChoiceDeb1[i]);
    InitChoiceDeb(g_globals_ui->ChoiceDeb2[i]);
    InitChoiceOnMode(g_globals_ui->ChoiceOnMode[i]);
  }
  FillOutGlobals();
  /* show the first form */
  fl_show_form(g_globals_ui->FormGlobals,
	       FL_PLACE_CENTERFREE,FL_FULLBORDER,"PIC Globals");
}
