#define _XOPEN_SOURCE 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>

#include "forms.h"
#include "board_ui.h"
#include "board.h"

static FD_FormSample *g_sample_ui;

static int g_avg[4]={0,0,0,0};
static XPoint *g_xpoint = NULL;
static struct_sample4 *g_sample4_array;
static Drawable g_win;
static Pixmap g_pixmap;
static GC g_gc;
static unsigned int g_depth;
static _byte g_shm;
static XShmSegmentInfo g_shminfo;
static FL_OBJECT *g_cvs;

#define BLACK   0x000000
#define RED     0xff0000
#define RED_B   0xff4040
#define GREEN   0x00ff00
#define BLUE    0x0000ff
#define YELLOW  0xffff00
#define MAGENTA 0xff00ff
#define CYAN    0x00ffff

static unsigned char g_shift = 0;

static unsigned int g_caption_colors[4] = {
  RED_B,GREEN,BLUE,MAGENTA
};

static void Cvs_SetColor(long color) {
  XSetForeground(fl_display, g_gc, color);
}

static void Cvs_Clear(void) {
  Cvs_SetColor(BLACK);
  XFillRectangle(fl_display,  g_pixmap, g_gc,
		 0, 0, g_cvs->w, g_cvs->h);
}

static void Cvs_DrawBaseLine(long color) {
  Cvs_SetColor(color);
  XDrawLine(fl_display, g_pixmap, g_gc, 
	    0, g_cvs->h>>1, g_cvs->w, g_cvs->h>>1);
}

static void Cvs_Refresh(void) {
  XCopyArea(fl_display, g_pixmap, g_win,
	    g_gc, 0, 0, g_cvs->w, g_cvs->h, 0, 0);
}

static int SignedShift(int val, unsigned char shift) {
  return(val < 0 ? -((-val)>>shift) : val>>shift);
}
static void Sample2XPoint(int sample_num) {
  unsigned int i, base = g_sample4_array[0].stamp;
  struct_sample4 *sample4_ptr = g_sample4_array+0;
  XPoint *xpoint_ptr = g_xpoint+0;
  for (i = 0; i < g_cvs->w; i++, sample4_ptr++, xpoint_ptr++) {
    xpoint_ptr->x = sample4_ptr->stamp-base;
    xpoint_ptr->y = (g_cvs->h>>1)+SignedShift(g_avg[sample_num]-
					      sample4_ptr->sample[sample_num],
					      g_shift);
  }
}

static void Cvs_Redraw(void) {
  unsigned char i;
  Cvs_Clear();
  Cvs_DrawBaseLine(BLUE);
  for (i = 0; i < 4; i++)
    if (fl_get_button(g_sample_ui->BtnView[i])) {
      Cvs_SetColor(g_caption_colors[i]);
      Sample2XPoint(i);
      XDrawLines(fl_display, g_pixmap, g_gc, g_xpoint,
		 g_cvs->w, CoordModeOrigin);
    }
  Cvs_Refresh();
  XSync(fl_display, False);
}

static int CanvasExposeHandler(FL_OBJECT *ob, Window win, 
			       int win_width, int win_height,
			       XEvent *xev, void *user_data) {
  Cvs_Redraw();
  return(0);
}

static void Cvs_Init(void) {
  int i;
  XImage *ximage;
  g_cvs = g_sample_ui->CanvasAccel;
  g_win = FL_ObjWin(g_cvs);
  g_gc = fl_state[fl_get_vclass()].gc[8];
  g_depth = fl_state[fl_get_vclass()].depth;
  g_shm = XShmQueryExtension(fl_display);
  printf("depth=%d, shm: %s\n", g_depth, g_shm?"ok":"not ok");
  if (g_shm) {
    ximage = XShmCreateImage(fl_display,
			     fl_state[fl_get_vclass()].xvinfo->visual,
			     g_depth,ZPixmap,NULL,
			     &g_shminfo,g_cvs->w,g_cvs->h);
    g_shminfo.shmid = shmget(IPC_PRIVATE,
			     ximage->bytes_per_line*ximage->height,
			     IPC_CREAT | 0777);
    g_shminfo.shmaddr = shmat(g_shminfo.shmid, 0, 0);
    g_shminfo.readOnly = False;
    XShmAttach(fl_display, &g_shminfo);
    g_pixmap = XShmCreatePixmap(fl_display,g_win,g_shminfo.shmaddr,
				&g_shminfo,g_cvs->w,g_cvs->h,g_depth);
  } else
    g_pixmap = XCreatePixmap(fl_display, g_win,
			      g_cvs->w, g_cvs->h, g_depth);
  fl_add_canvas_handler(g_cvs, Expose, CanvasExposeHandler, NULL);
  g_xpoint = (XPoint *)malloc(sizeof(XPoint)*g_cvs->w);
  for (i=0; i<g_cvs->w; i++)
    g_xpoint[i].x = g_xpoint[i].y = 0;
  g_sample4_array =
    (struct_sample4 *)malloc(g_cvs->w*sizeof(struct_sample4));
  Board_GetSamples(g_cvs->w, g_sample4_array);
  Cvs_Redraw();
}

/* EXPORTED */

void CbkRedraw(FL_OBJECT *ob, long data) {
  Cvs_Redraw();
}

void CbkCounterShift(FL_OBJECT *ob, long data) {
  g_shift = (unsigned char)fl_get_counter_value(g_sample_ui->CounterShift);
  Cvs_Redraw();
}

void CbkBtnSampleGraph(FL_OBJECT *ob, long data) {
  Board_GetSamples(g_cvs->w, g_sample4_array);
  Cvs_Redraw();
}

void CbkBtnSampleStats(FL_OBJECT *ob, long data) {
  int i, j;
  int n = atoi(fl_get_input(g_sample_ui->InputQty));
  if (n<1) {
    for (i=0; i<4; i++) {
      fl_set_object_label(g_sample_ui->TextSampleAvg[i], "XXX");
      fl_set_object_label(g_sample_ui->TextSampleStdDev[i], "XXX");
    }
  } else {
    char buf[32];
    double sample, sum_x[4], sum_x2[4], avg[4], std_dev[4];
    double n_inv = 1.0/(double)n;
    struct_sample4 *sample4_array =
      (struct_sample4 *)malloc(n*sizeof(struct_sample4));
    Board_GetSamples(n, sample4_array);
    for(i=0; i<4; i++) {
      sum_x[i] = 0.0; sum_x2[i] = 0.0;
    }
    for (i=0; i<n; i++)
      for (j=0; j<4; j++) {
	sample = (double)(sample4_array[i].sample[j]);
	sum_x[j] += sample; sum_x2[j] += sample*sample;
      }
    for (i=0; i<4; i++) {
      avg[i] = n_inv*sum_x[i];
      std_dev[i] = sqrt(n_inv*sum_x2[i] - avg[i]*avg[i]);
    }
    for (i=0; i<4; i++) {
      sprintf(buf, "%.2f", avg[i]);
      fl_set_object_label(g_sample_ui->TextSampleAvg[i], buf);
      sprintf(buf, "%.2f", std_dev[i]);
      fl_set_object_label(g_sample_ui->TextSampleStdDev[i], buf);
    }
    for (i=0; i<4; i++)
      g_avg[i] = (int)avg[i];
    free(sample4_array);
  }
}

void FormSample_Init(void) {
  g_sample_ui = create_form_FormSample();
  fl_set_input(g_sample_ui->InputQty, "640");
  fl_set_input_return(g_sample_ui->InputQty, FL_RETURN_END);
  fl_set_counter_step(g_sample_ui->CounterShift, 1.0, 1.0);
  fl_set_counter_value(g_sample_ui->CounterShift, (double)g_shift);
  fl_show_form(g_sample_ui->FormSample,
	       FL_PLACE_CENTERFREE,FL_FULLBORDER,"Sample Viewer");
  CbkBtnSampleStats(NULL, 0);
  Cvs_Init();

}
