/* (C) Dan S. Reznik - UCB EECS */

#define VERSION_INFO "Firmware V-0.7 - (C) Dan Reznik, 3/25/99"

#define ISR_FREQ 5000
#define SEMICYCLE_SAMPLES_EXP 5
#define SEMICYCLE_SAMPLES (1<<SEMICYCLE_SAMPLES_EXP)
#define SEMICYCLE_MASK (SEMICYCLE_SAMPLES-1)
#define CYCLE_SAMPLES (SEMICYCLE_SAMPLES<<1)
#define CYCLE_MASK (CYCLE_SAMPLES-1)

/* TODO: Make OP_Set_* change value during chirping or off phase */

typedef enum {
  ON_MODE_SUSTAIN,ON_MODE_PULSE_ONCE,ON_MODE_PULSE_TRAIN
} enum_ON_MODE;

typedef enum {
  DEB_WAVE,DEB_ENVELOPE,DEB_SIGNAL
} enum_DEB; /* selects waveform at output */

typedef enum {
  ENV_INIT,
  ENV_ATTACK_1,ENV_ATTACK_2,ENV_SUSTAIN,ENV_DECAY_1,ENV_DECAY_2,
  ENV_OFF
} enum_ENV;

typedef enum {
  SHIFT_UP_MID,
  SHIFT_DN_MID,
  SHIFT_UP_TOP,
  SHIFT_DN_TOP
} enum_SHIFT;

typedef enum {
  OP_On,
  OP_Off,
  OP_Inc,          /* echoes incremented byte */
  OP_Sample,       /* gets 4 11-bit samples */
  OP_Sample_Batch, /* gets many sample groups */
  OP_Set_Envelope, /* attack, sustain, shift, shift_mode */
  OP_Set_Wave,     /* phase, mult1, mult2 */
  OP_Set_Steps,    /* step_1, step_2 */
  OP_Set_Debug,    /* debug_1, debug_2 */
  OP_Set_Freq,
  OP_Set_On_Mode,  /* on_mode */
  OP_Get_State,    /* envelope state */    
  OP_Get_Glob,
  OP_Get_Version_Info,
  OP_NUM
} enum_OP;

typedef struct {
  unsigned short stamp;
  signed short step_1, step_2;
  unsigned short attack, sustain; /* in 1/ISR_FREQ units */
  /* list all even-sized globals above to avoid char word alignemnt */
  unsigned char shift, /* enum_SHIFT */ shift_mode;
  unsigned char state; /* enum_ENV */
  unsigned char phase, mult1, mult2; /* mult1*sin[x-phase] + mult2*sin[2x] */
  unsigned char on_mode; /* enum_ON_MODE */
  unsigned char deb_1, deb_2; /* enum_DEB */
  unsigned char freq; /* actual_freq = (ISR_FREQ/CYCLE_SAMPLES)*(freq+1)/256 */
} struct_glob;
