#include <stdio.h>
#include "board.h"

static void Print_Struct_Glob(struct_glob *glob_ptr) {
  static char *debug_name[] = {"sampl", "envel", "signl"}; 
  static char *state_name[] = {"init", "att_1", "att_2", "sust",
			       "dec_1", "dec_2", "off"}; 
  printf("%6s%6s%6s%6s%6s%6s%6s%6s%6s%6s%6s%6s\n",
	 "stamp","phase","mult1","mult2",
	 "att","sust","st_1","st_2",
	 "shift","state","deb_1","deb_2");
  printf("%6d%6d%6d%6d%6d%6d%6d%6d%6d%6s%6s%6s\n",
	 glob_ptr->stamp,glob_ptr->phase,
	 glob_ptr->mult1,glob_ptr->mult2,
	 glob_ptr->attack_periods,glob_ptr->sustain_periods,
	 glob_ptr->envelope_step_1,glob_ptr->envelope_step_2,
	 glob_ptr->envelope_shift,
	 state_name[glob_ptr->envelope_state],
	 debug_name[glob_ptr->deb_1],debug_name[glob_ptr->deb_2]);
}

static void Print_Samples(char *fname, int sample_num, struct_sample4 *sample4_ptr) { 
  int i;
  FILE *fp = fopen(fname, "w");
  for (i = 0; i < sample_num; i++, sample4_ptr++)
    fprintf(fp, "%d %d %d %d %d %d\n",
	    i, sample4_ptr->stamp,
	    sample4_ptr->sample[0],sample4_ptr->sample[1],
	    sample4_ptr->sample[2],sample4_ptr->sample[3]);
  fclose(fp);
}

static void Print_Glob(void) {
  struct_glob glob_a, glob_b;
  Board_GetGlob(&glob_a, &glob_b);
  printf("\nPIC_A\n"); Print_Struct_Glob(&glob_a);
  printf("\nPIC_B\n"); Print_Struct_Glob(&glob_b);
}

#define SAMPLE_NUM 100

int main(int argc, char **argv) {
  struct_sample4 sample4_array[SAMPLE_NUM];
  struct_wave wave_a, wave_b;
  struct_envelope env_a, env_b;
  enum_DEB deb[4];
  short intensity[4];
  if (Board_Init() == -1)
    return(-1);
  Print_Glob();
  Board_GetSamples(SAMPLE_NUM, sample4_array);
  Print_Samples("samples.out", SAMPLE_NUM, sample4_array);
  wave_a.phase = 01; wave_a.mult1 = 02; wave_a.mult2 = 03;
  wave_b.phase = 11; wave_b.mult1 = 22; wave_b.mult2 = 33;
  Board_SetWaveParams(&wave_a, &wave_b);
  printf("\nAfter Board_SetWaveParams():\n"); Print_Glob();
  env_a.attack_periods = 5; env_a.sustain_periods = 6; env_a.shift = 7;
  env_b.attack_periods = 2; env_b.sustain_periods = 3; env_b.shift = 4;
  Board_SetEnvelopeParams(&env_a, &env_b);
  printf("\nAfter Board_SetEnvelopeParams():\n"); Print_Glob();
  deb[0] = DEB_SAMPLE;   deb[1] = DEB_ENVELOPE;
  deb[2] = DEB_ENVELOPE; deb[3] = DEB_SAMPLE;
  Board_SetDebug(deb);
  printf("\nAfter Board_SetDebug():\n"); Print_Glob();
  intensity[0] = -11; intensity[1] = -22;
  intensity[2] = 33; intensity[3] = 44;
  Board_On(intensity, 0);
  Board_Off();
  printf("\nAfter Board_On( --pulse-- ):\n"); Print_Glob();

  intensity[0] = -88; intensity[1] = -77;
  intensity[2] = 99; intensity[3] = 111;
  Board_On(intensity, 1);
  Board_Off();
  printf("\nAfter Board_On( --continuous-- ):\n"); Print_Glob();

  /* RESET */
  Board_Reset();
  printf("\nAfter Board_Reset():\n"); Print_Glob();
  return(0);
}
