#include <stdio.h>

#ifdef PPUSER
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/parport.h>
#include "ppuser.h"
#else
#include <sys/io.h>
#define DATA 0x378
#define STAT (DATA+1)
#define CTRL (DATA+2)
#endif

#include "board_proto.h"

#ifdef PPUSER
static int g_port;
#endif

int Parallel_Init(void) {
#ifdef PPUSER
  g_port = open("/dev/ppuser00", O_RDWR);
  if (g_port == -1) {
    fprintf(stderr, "ERROR: can't open /dev/ppuser00\n");
    return(-1);
  }
  if (ioctl(g_port, PPCLAIM, 0)) {
    fprintf(stderr, "ERROR: can't claim pport\n");
    return(-1);
  }
  return(0);
#else
  if ( (ioperm(DATA, 3, 1)==-1) ) {
    fprintf(stderr, "ERROR: cannot open I/O.\n");
    return(-1);
  } else
    return(0);
#endif
}

void Parallel_Write_Data(_byte b) {
#ifdef PPUSER
  if (ioctl(g_port, PPWDATA, &b))
    fprintf(stderr, "ERROR: Parallel_Write_Data()\n");
#else
  outb(b,DATA);
#endif
}

void Parallel_Write_Status(_byte b) {
#ifdef PPUSER
  if (ioctl(g_port, PPWSTATUS, &b))
    fprintf(stderr, "ERROR: Parallel_Write_Status()\n");
#else
  outb(b,STAT);
#endif
}

void Parallel_Write_Control(_byte b) {
#ifdef PPUSER
  if (ioctl(g_port, PPWCONTROL, &b))
    fprintf(stderr, "ERROR: Parallel_Write_Control()\n");
#else
  outb(b,CTRL);
#endif
}

_byte Parallel_Read_Data(void) {
#ifdef PPUSER
  _byte b=0;
  if (ioctl(g_port, PPRDATA, &b))
    fprintf(stderr, "ERROR: Parallel_Read_Data()\n");
  return(b);
#else
  return(inb(DATA));
#endif
}

_byte Parallel_Read_Status(void) {
#ifdef PPUSER
  _byte b=0;
  if (ioctl(g_port, PPRSTATUS, &b))
    fprintf(stderr, "ERROR: Parallel_Read_Status()\n");
  return(b);
#else
  return(inb(STAT));
#endif
}

_byte Parallel_Read_Control(void) {
#ifdef PPUSER
  _byte b=0;
  if (ioctl(g_port, PPRCONTROL, &b))
    fprintf(stderr, "ERROR: Parallel_Read_Control()\n");
  return(b);
#else
  return(inb(CTRL));
#endif
}
